from atexit import register
from datetime import datetime
import sys, psutil, os


def alertExit() -> None:
    info = psutil.virtual_memory()
    print(u'内存使用：',psutil.Process(os.getpid()).memory_info().rss)
    print(u'总内存：',info.total)
    print(u'内存占比：',info.percent)
    print(u'cpu个数：',psutil.cpu_count())
    print("感谢使用！", datetime.now())
    print("-"*30)
#     dealExit()


# @register
# def dealExit() -> str:
#     sys.exit(0)





from __future__ import absolute_import
from .zjwbox import Infer
from .auto_req import get_window_page, get_source_page, extra
from .comm_req import Comm_req
from .update import update_driver
from .store import Store
from .auto_sel import AutoSel
from .pre_deal import KnnMiss
from pyppeteer import chromium_downloader


name = "zjwbox"
version = "0.3.2"


pyppeteer_version_win32 = chromium_downloader.downloadURLs.get("win32")
pyppeteer_version_win64 = chromium_downloader.downloadURLs.get("win64")




from selenium import webdriver
from selenium.webdriver.common.action_chains import ActionChains
from dataclasses import dataclass
from tqdm import tqdm, trange
import json
from datetime import datetime


@dataclass
class AutoSel(object):
    driver_path: str
    urls: str or list = None
    browser_type: str = None
    css_args: list or tuple = None

    def get_source_page(self, urls: str or list, css_args: list or tuple=None) -> str:
        driver = webdriver.Chrome(executable_path=self.driver_path)
        try:
            driver.get(urls)
            text = driver.page_source
            page_info = driver.get_cookies()
            driver.close()
        except Exception as e:
            print(e)
            driver.close()
        else:
            return text, page_info

    def get_window_page(self, urls: str or list, css_args: list or tuple=None) -> str:
        driver = webdriver.Chrome(executable_path=self.driver_path)
        try:
            driver.get(urls)
            p_texts = []
            p_text = driver.find_elements_by_css_selector(" div > div > p")
            p_text_tqdm = tqdm(p_text)
            for p in p_text_tqdm:
                p_text_tqdm.set_description("正在进行中...")
                if p.text.strip() == '':
                    continue
                p_texts.append(p.text.strip())

        except Exception as e:
            print(e)
            driver.close()
        else:
            return p_texts






# if __name__ == "__main__":
#     driver_path = "D:\\chromedriver\\chromedriver.exe"
#     url = "https://wenku.baidu.com/view/7bb441e6a88271fe910ef12d2af90242a995ab1c.html?fr=aladdin664466&ind=1"
#     a = AutoSel(driver_path=driver_path)
#     text = a.get_window_page(url)
#     print(text)


from zjwbox.encrypt.login_all import LoginAll
import re


lg = LoginAll()
register_dict = {
    "lagou": lg.lagou,
    "douban": lg.douban,
    "taobao": lg.taobao,
    "zhihu": lg.zhihu,
    "jingdong": lg.jingdong,
    "baidupan": lg.baidupan,
    "QQZone": lg.QQZone,
    "ifeng": lg.ifeng,
    "eSurfing": lg.eSurfing,
    "zgconline": lg.zgconline,
    "github": lg.github,
    "zt12306": lg.zt12306,
    "music163": lg.music163,
    "sohu": lg.sohu,
    "bilibili": lg.bilibili,
    "weibo": lg.weibo,
    "youdao": lg.youdao,
    "renren": lg.renren
}


def GetSession(url):
    pattern = r"[http, https]://\w+.(.*?).com"
    domain = re.findall(string=url, pattern=pattern)[0]
    session = register_dict[domain]
    return session


import logging
import sys


class LogSet(object):
    def __init__(self, mod_name=None):
        self.mod_name = __name__

    def debug_log(self, levelname: str) -> all:
        levels = {
            "error": logging.DEBUG,
            "warn": logging.INFO,
            "debug": logging.WARNING,
            "no": logging.ERROR
        }
        logging.basicConfig(
            level=levels[levelname],
            format="\033[1;31m%(message)s\n%(asctime)s - 行数：%(lineno)d - 模块：%(module)s\033[0m"
        )
        logger = logging.getLogger(self.mod_name)
        return logger

    def output_log(self, levelname: str) ->all :
        levels = {
            "info": logging.INFO,
            "debug": logging.DEBUG,
            "warn": logging.WARNING,
            "error": logging.ERROR
        }
        logging.basicConfig(
            level=levels[levelname],
            format=f"%(asctime)s - {sys.argv[0]} - 模块：%(module)s - 行数：%(lineno)d - %(message)s",
            filename="output.log"
        )
        logger = logging.getLogger(self.mod_name)
        return logger


log = LogSet()
l = log.output_log(levelname="debug")
record = l.debug
record("debug")



from DecryptLogin import login
from selenium import webdriver


lg = login.Login()



class LoginAll(object):
    def __init__(self):
        ...

    # 拉钩
    def lagou(self, username: str, password: str) -> all:
        return_info, session = lg.lagou(username, password, "pc")
        return session

    # 人人网
    def renren(self, username: str, password: str) -> all:
        return_info, session = lg.renren(username, password, "pc")
        return session

    # 豆瓣
    def douban(self, username: str, password: str) -> all:
        return_info, session = lg.douban(username, password)
        return session

    # 知乎
    def zhihu(self, username: str, password: str) -> all:
        return_info, session = lg.zhihu(username, password)
        return session

    # 有道
    def youdao(self, username: str, password: str) -> all:
        return_info, session = lg.youdao(username, password, "pc")
        return session

    # 微博
    def weibo(self, username: str, password: str) -> all:
        return_info, session = lg.weibo(username, password, "pc")
        return session

    # 哔哩哔哩
    def bilibili(self, username: str, password: str) -> all:
        return_info, session = lg.bilibili(username, password, "pc")
        return session

    # 搜狐
    def sohu(self, username: str, password: str) -> all:
        return_info, session = lg.sohu(username, password)
        return session

    # 网易云
    def music163(self, username: str, password: str) -> all:
        return_info, session = lg.music163(username, password)
        return session

    # 中国铁路12306
    def zt12306(self, username: str, password: str) -> all:
        return_info, session = lg.zt12306(username, password)
        return session

    # github
    def github(self, username: str, password: str) -> all:
        return_info, session = lg.github(username, password)
        return session

    # 中国在线中关村
    def zgconline(self, username: str, password: str) -> all:
        return_info, session = lg.zgconline(username, password)
        return session

    # 天翼
    def eSurfing(self, username: str, password: str) -> all:
        return_info, session = lg.eSurfing(username, password)
        return session

    # 凤凰网
    def ifeng(self, username: str, password: str) -> all:
        return_info, session = lg.ifeng(username, password)
        return session

    # QQ空间
    def QQZone(self, username: str, password: str) -> all:
        return_info, session = lg.QQZone(username, password)
        return session

    # 百度网盘
    def baidupan(self, username: str, password: str) -> all:
        return_info, session = lg.baidupan(username, password)
        return session

    # 淘宝
    def taobao(self, username: str, password: str) -> all:
        return_info, session = lg.taobao(username, password)
        return session

    # 京东
    def jingdong(self, username: str, password: str) -> all:
        return_info, session = lg.jingdong(username, password)
        return session


if __name__ == "__main__":
    e = LoginAll()
    session = e.jingdong("17685287506", "218347800wei")
    print(session)
    url = "https://www.renren.com"
    # res = session.get(url)
    # text = res.text
    # print(text)

    # print(cookie)
    # url = "https://www.douban.com"
    # driver = webdriver.Chrome("D:\\chromedriver\\chromedriver.exe")
    # driver.add_cookie(cookie)
    # driver.get(url)
    # driver.implicitly_wait(3)
    # input()


from pyquery import PyQuery as pq
import re
from requests.exceptions import ConnectionError
from fake_useragent import UserAgent, FakeUserAgentError
import requests
import random
import time, json
import threading, asyncio
import eventlet
from zjwbox.boxtools.log_set import LogSet


logger = LogSet().debug_log(levelname="debug")
log = LogSet().output_log(levelname="debug")
debug = logger.warning
warn = logger.info
error = logger.debug
record = log.info

def get_page(url, options={}):
    try:
        ua = UserAgent()
    except FakeUserAgentError:
        pass
    # 生成随机的请求头，加 try...except... 使代码更加健壮
    base_headers = {
        'User-Agent': ua.random,
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'zh-CN,zh;q=0.8'
    }
    # 如果使用者有传入请求头，则将此请求头和随机生成的合成在一起
    headers = dict(base_headers, **options)
    # 当前请求的 url
    print('Getting', url)
    try:
        r = requests.get(url, headers=headers)
        print('Getting result', url, r.status_code)
        if r.status_code == 200:
            return r.text
        return None
    except ConnectionError:
        print('Crawling Failed', url)
        return None


class ProxyMetaclass(type):

    def __new__(cls, name, bases, attrs):
        count = 0
        attrs['__CrawlFunc__'] = []
        for k, v in attrs.items():
            if 'crawl_' in k:
                attrs['__CrawlFunc__'].append(k)
                count += 1
        attrs['__CrawlFuncCount__'] = count
        return type.__new__(cls, name, bases, attrs)


class FreeProxyGetter(object, metaclass=ProxyMetaclass):
    """
    免费代理获取
    1. 快代理
    2. 西拉代理
    3. 66代理
    4. 无忧代理
    5. 开心代理-高匿
    """

    def get_raw_proxies(self, callback):
        proxies = []
        print('Callback', callback)
        for proxy in eval("self.{}()".format(callback)):
            print('Getting', proxy, 'from', callback)
            proxies.append(proxy)
        return proxies

    def crawl_kuaidaili(self):
        """
        快代理
        :return:
        """
        for page in range(1, 4):
            start_url = 'https://www.kuaidaili.com/free/inha/{}/'.format(page)
            html = get_page(start_url)
            pattern = re.compile(
                '<td data-title="IP">(.*)</td>\s*<td data-title="PORT">(\w+)</td>'
            )

            ip_addres = pattern.findall(str(html))
            for adress, port in ip_addres:
                # print(adress, port)
                result = f"{adress}:{port}".strip()
                yield result

    def crawl_xicidaili(self):
        """
        西拉代理
        :return:
        """
        for page in range(1, 4):
            start_url = 'http://www.xiladaili.com/'
            html = get_page(start_url)
            pattern = r"<>(\d+.\d+.\d+.\d+:\d+)<>"
            ip_adress = re.findall(string=html, pattern=pattern)
            for ip in ip_adress:
                result = ip.strip()
                yield result

    def crawl_daili66(self, page_count=4):
        """
        66代理
        :param page_count:
        :return:
        """
        start_url = 'http://www.66ip.cn/{}.html'
        urls = [start_url.format(page) for page in range(1, page_count + 1)]
        for url in urls:
            print('Crawling', url)
            record('Crawling' + url)
            html = get_page(url)
            if html:
                doc = pq(html)
                trs = doc('.containerbox table tr:gt(0)').items()
                for tr in trs:
                    ip = tr.find('td:nth-child(1)').text()
                    port = tr.find('td:nth-child(2)').text()
                    yield ':'.join([ip, port])

    def crawl_data5u(self):
        """
        无忧代理
        :return:
        """
        start_url = 'http://www.data5u.com'
        html = get_page(start_url)
        # print(html)
        ip_adress = re.compile(
            '<ul class="l2">\s*<span><li>(.*?)</li></span>\s*<span style="width: 100px;"><li class=".*">(.*?)</li></span>'
        )
        # \s * 匹配空格，起到换行作用
        re_ip_adress = ip_adress.findall(str(html))
        for adress, port in re_ip_adress:
            result = f"{adress}:{port}"
            yield result.strip()

    def crawl_kxdaili(self):
        """
        开心代理-高匿
        :return:
        """
        for i in range(1, 4):
            start_url = 'http://www.kxdaili.com/dailiip/1/{}.html'.format(i)
            try:
                html = requests.get(start_url)
                if html.status_code == 200:
                    html.encoding = 'utf-8'
                    # print(html.text)
                    ip_adress = re.compile('<tr.*?>\s*<td>(.*?)</td>\s*<td>(.*?)</td>')
                    # \s* 匹配空格，起到换行作用
                    re_ip_adress = ip_adress.findall(str(html.text))
                    for adress, port in re_ip_adress:
                        result = f"{adress}:{port}"
                        yield result.strip()
                return None
            except:
                pass


def kuaidaili():
    result = []
    proxy = FreeProxyGetter()
    kuaidailis = proxy.crawl_xicidaili()

    for kuaidaili in kuaidailis:
        result.append(kuaidaili)
    return result


def xicidaili():
    result = []
    proxy = FreeProxyGetter()
    xicidailis = proxy.crawl_xicidaili()

    for xicidaili in xicidailis:
        result.append(xicidaili)
    return result


def data5u():
    result = []
    proxy = FreeProxyGetter()
    data5us = proxy.crawl_data5u()

    for data5u in data5us:
        result.append(data5u)
    return result


def kxdaili():
    result = []
    proxy = FreeProxyGetter()
    kxdailis = proxy.crawl_kxdaili()

    for kxdaili in kxdailis:
        result.append(kxdaili)
    return result


def daili66():
    result = []
    proxy = FreeProxyGetter()
    daili66s = proxy.crawl_daili66()
    kxdailis = proxy.crawl_kxdaili()

    for daili66 in daili66s:
        result.append(daili66)
    for kxdaili in kxdailis:
        result.append(kxdaili)
    return result


class MyThread(threading.Thread):

    def __init__(self, func, args=()):
        super(MyThread, self).__init__()
        self.func = func
        self.args = args

    def run(self):
        self.result = self.func(*self.args)

    def get_result(self):
        try:
            return self.result   # 如果子线程不使用join方法，此处可能会报没有self.result的错误
        except Exception:
            return None

def allproxy():
    result = []
    proxy = FreeProxyGetter()

    daili66s = proxy.crawl_daili66()
    data5us = proxy.crawl_data5u()
    kxdailis = proxy.crawl_kxdaili()
    xicidailis = proxy.crawl_xicidaili()
    kuaidailis = proxy.crawl_kuaidaili()

    for kuaidaili in kuaidailis:
        result.append(kuaidaili)

    for xicidaili in xicidailis:
        result.append(xicidaili)

    for kxdaili in kxdailis:
        result.append(kxdaili)

    for data5u in data5us:
        result.append(data5u)

    for daili66 in daili66s:
        result.append(daili66)

    return result

# return of result
use_proxy = []


async def usefulProxy(i):
    try:

        ip_proxy = i  # random.choice(http_ip)
        proxy_ip = {
            'http': ip_proxy,
            'https': ip_proxy,
        }
        print('使用代理的IP:', proxy_ip)
        response = requests.get("http://httpbin.org/ip", proxies=proxy_ip, timeout=1).text
        print('\033[1;32m当前IP有效\033[0m')
        record('\033[1;32m当前IP有效\033[0m')
        use_proxy.append(proxy_ip)
        with open("getProxy.txt", mode="w", encoding="utf-8") as fg:
            fg.write( json.dumps(proxy_ip) )
        return
    except Exception as e:
        print(e.args[0])
        print('\033[1;31m当前IP无效\033[0m')
        record(e.args[0])
        record('\033[1;31m当前IP无效\033[0m')


async def get_task():
    all_proxy = allproxy()
    tasks = [asyncio.ensure_future(usefulProxy(i)) for i in all_proxy]
    await asyncio.gather(*tasks)


def get_proxy():
    # asyncio deal
    loop = asyncio.get_event_loop()
    loop.run_until_complete(get_task())

    eventlet.monkey_patch()
    while not use_proxy:
        print("没有可用的IP，准备重启中...")
        with eventlet.Timeout(5, False):
            for i in range(5,0,-1):
                print(f"\r正在重启，倒计时{i+1}秒...", flush=True, end="")
                time.sleep(1)
        get_proxy()

    return use_proxy



# a example of !
if __name__ == "__main__":
    a = get_proxy()
    print(a)



import requests
import parsel, re, json, jsonpath
import time, random, sys, psutil
import asyncio, aiohttp
from tqdm import tqdm
from itertools import product
import eventlet
from faker import Faker
from datetime import datetime
from zjwbox.boxtools.time_delay import DelayWait
from zjwbox.boxtools.log_set import LogSet
from zjwbox.boxtools.proxyShop import get_proxy
from zjwbox.boxtools.check_url import CheckUrl
from zjwbox.boxtools.get_session import GetSession
import os
# os.environ['NO_PROXY'] = 'stackoverflow.com'

"""
This is a person to solve common spider in website.
You may transmit some args to it.
Least need a arg(url or urls), 
of couser, you also can change many args, headers cookie, data, mode, etc.
Matters need attention: urls should be a string or list. 
"""

# data of faker
fake = Faker()
# product of user-agent
fake_ua = fake.user_agent()


class Comm_req(object):

    def __init__(self, urls: str or list, headers: dict={"UserAgent": fake_ua}, cookie: dict=None, data: dict=None, 
                        encode: str="utf-8", mode: str="get", time_inter: bool=True, debug_mode: str="no", is_proxy=False):
        self.urls = urls
        self.headers = headers
        self.cookie = cookie
        self.data = data
        self.mode = mode
        self.encode = encode
        self.time_inter = time_inter
        self.is_proxy = is_proxy
        # log
        self.logger = LogSet().debug_log(levelname=debug_mode)
        self.log = LogSet().output_log(levelname="debug")
        self.debug = self.logger.warning
        self.warn = self.logger.info
        self.error = self.logger.debug
        self.record = self.log.info


    async def fask(self, url, mode="get") -> str:
        async with aiohttp.ClientSession() as session:
            if self.mode == "post":
                async with session.post(url, headers=self.headers, data=self.data, timeout=3) as res:
                    return await res.text()
            else:
                async with session.get(url, headers=self.headers, timeout=3) as res:
                    return await res.text()

    def get_proxys(self) -> dict:
        get_proxy()
        with open("getProxy.txt", mode="r", encoding="utf-8", errors='ignore') as f:
            # ip = f.read()
            return json.load(f, strict=False)


    def comm_deal(self, expr_element: list or tuple or str, is_json: bool=False, session=requests.Session()) -> str:
        if isinstance(self.urls, str):
            if self.time_inter:
                res = session.get(self.urls, headers=self.headers, timeout=5)
                text = res.content.decode(self.encode)
                if self.is_proxy and self.mode == "get":
                    proxies = self.get_proxys()
                    try:
                        res = session.get(self.urls, headers=self.headers, timeout=5, proxies=proxies)
                        text = res.content.decode(self.encode)
                    except requests.exceptions.ReadTimeout as e:
                        print("ip失效！")
                        return None
                if self.mode == "post":
                    res = session.post(self.urls, headers=self.headers, data=self.data)
                    text = res.content.decode(self.encode)
                    if self.is_proxy:
                        proxies = self.get_proxys()
                        try:
                            res = session.post(self.urls, headers=self.headers, data=self.data, proxies=proxies)
                            text = res.content.decode(self.encode)
                        except requests.exceptions.ReadTimeout as e:
                            print("ip失效！")
                            return None
                delay = DelayWait()
                delay.wait(self.urls)
                if is_json:
                    json_text = jsonpath.jsonpath(json.loads(text), f"$..{expr_element}")
                    self.debug("json_text：" + str(json_text))
                    self.record(json_text)
                    yield json_text
                else:
                    self.debug("text：" + str(text) )
                    self.record(text)
                    yield text

            else:
                try:
                    proxies = self.get_proxys()
                    res = session.get(self.urls, headers=self.headers, proxies=proxies, timeout=5)
                    self.debug("res：" + str(res))
                    self.record(res)
                    text = res.content.decode(self.encode)
                    self.debug("text：" + str(text))
                    self.record(text)
                except requests.exceptions.ReadTimeout as e:
                    print("ip失效！")
                    return None
                if self.is_proxy and self.mode == "get":
                    try:
                        proxies = self.get_proxys()
                        res = session.get(self.urls, headers=self.headers, timeout=5, proxies=proxies)
                    except requests.exceptions.ReadTimeout as e:
                        print("ip失效！")
                        return None       
                    else:             
                        self.debug("res：" + str(res))
                        self.record(res)
                        text = res.content.decode(self.encode)
                        self.debug("text：" + str(text))
                        self.record(text)
                if self.mode == "post":
                    res = session.post(self.urls, headers=self.headers, data=self.data)
                    self.debug("res：" + str(res))
                    self.record(res)
                    text = res.content.decode(self.encode)
                    self.debug("text：" + str(text))
                    self.record(text)
                    if self.is_proxy:
                        proxies = self.get_proxys()
                        try:
                            res = session.post(self.urls, headers=self.headers, data=self.data, proxies=proxies)
                            self.debug("res：" + str(res))
                            self.record(res)
                            text = res.content.decode(self.encode)
                            self.debug("text：" + str(text))
                            self.record(text)
                        except requests.exceptions.ReadTimeout as e:
                            print("ip失效！")
                            return None                            
                yield text

        else:
            for url in self.urls:
                loop = asyncio.get_event_loop()
                text = loop.run_until_complete( self.fask(url, self.mode) )
                self.debug("text：" + str(text))
                self.record(text)
                now_time = datetime.now()
                print(f"爬取{url}成功！时间：{now_time}")
                delay = DelayWait()
                delay.wait(url)
                yield text


    def get_page(self, expr_element: list or tuple or str="", is_json: bool=False, login_args: dict=None):
        result = []
        if CheckUrl(self.urls):
            session = GetSession(self.urls)(login_args["username"], login_args["password"])
            for page in self.comm_deal(expr_element, is_json=is_json, session=session):
                result.append(page)

        for page in self.comm_deal(expr_element, is_json=is_json):
            result.append(page)
        if result == []:
            return None
        eventlet.monkey_patch()
        now_time = datetime.now()
        if isinstance(self.urls, str):
            print(f"目标1个，成功爬取{len(result)}, 时间：{now_time}")
        else:
            print(f"目标{len(self.urls)}个，成功爬取{len(result)}，时间：{now_time}")
  
        return result
        




if __name__ == "__main__":
    url = "http://hgk.guizhou.gov.cn/gzdata/getGroupTableData.action"
    data = {
        "tableId": 725,
        "year_limit": 1,
        "address_code": "0001",
        "field_code": "0002"
    }



# -*- coding: utf-8 -*-
from urllib import parse
from datetime import datetime
import time, random


rand_delay = random.uniform(0.9, 1.5)

class DelayWait(object):
    def __init__(self, delay=rand_delay):
        self.delay = delay
        self.urls = dict()
        # 重复次数
        self.count = 0

    def wait(self, url: str) -> None:
        netloc = parse.urlparse(url).netloc
        lastOne = self.urls.get(netloc)
        # 是否已请求过
        if lastOne and self.delay > 0:
            # 两次时间差
            timeWait = self.delay - (datetime.now() - lastOne).seconds
            if timeWait > 0:
                if self.count > 4 and self.count < 15:
                    time.sleep(timeWait * 2)
                    self.count += 1
                elif self.count > 15:
                    time.sleep(timeWait * 3)
                    self.count += 1
                else:
                    time.sleep(timeWait)
                    self.count += 1
        self.urls[netloc] = datetime.now()




from dataclasses import dataclass
import pymongo, pymysql, redis
from tqdm import tqdm, trange
import csv, openpyxl
from collections.abc import Iterable
from threading import Lock
from zjwbox.boxtools.log_set import LogSet
from datetime import datetime



"""
welcome you !
This is a class about store data !
You might use it store many different type for your data
For example, txt, Csv, Excel, a kind of Databases !
You should give it some args, text of data, path of file and name of file,
It is necessary to some args, your data in text, path of file, filename.
"""


@dataclass
class Store(object):
    text: list or dict
    path: str
    filename:str
    debug_mode: str = "no"
    logger: object = LogSet().debug_log(levelname=debug_mode)
    log: object = LogSet().output_log(levelname="debug")
    debug: object = logger.warning
    warn:object = logger.info
    error:object = logger.debug
    record:object = log.info

    _instance_lock = Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(Store, "_instance"):
            with Store._instance_lock:
                if not hasattr(Store, "_instance"):
                    Store._instance = object.__new__(cls)  
        return Store._instance


    def __str__(self) -> str:
        info = """
                This is a class about store data !
                You able use it store many different type for your data !
                For example, txt, Csv, Excel, a kind of Databases !
                """
        return info


    """
    Save to your TXT !
    you may give some choice args, for example,  encode of char, 
    even can change is_mark that decide read, write or add to !
    """
    def to_txt(self, encode: str="utf-8", is_mark=True ) -> all:
        if isinstance(self.text, list):
            words, count = 0, 0
            mode = "a+"
            if is_mark:
                mode = "w+"
            with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode) as f:
                for i in tqdm(self.text):
                    f.writelines(i)
                    count += 1
                    words += len(i)
                    self.debug(words)
                    self.record(words)
            print(f"保存完成！location of file：{self.path}/{self.filename}，共{count}行，{words}字！")
        else:
            print("TypeErorr: text must be list !")


    """
    Save to your CSV !
    you can give some choice args for this function, for example, mode to read and write, 
    if your datas is dict or set of dict, you may give a arg of headers(title in csv),
    you even can change is_single that deal most datas !
    """
    def to_csv(self, mode="w+", encode: str="utf-8", is_single=True, headers=None)-> None :
        if ( isinstance(self.text, list) and not isinstance(self.text[0], dict) ) or isinstance(self.text[0], list):
            if mode == "r":
                with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode) as f:
                    reader = csv.reader(f)
                    for row in reader:
                        print(row)
                        self.record(row)
            with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode, newline="") as f:
                writer = csv.writer(f)
                if isinstance(self.text[0], list):
                    if is_single:
                        for t in tqdm(self.text):
                            writer.writerow(t)
                            self.debug(t)
                            self.record(t)
                            tqdm(self.text).set_description("正在写入中！")
                elif isinstance(self.text, list):
                    if is_single:
                        for t in tqdm(self.text):
                            writer.writerow(t)
                            self.debug(t)
                            self.record(t)
                            tqdm(self.text).set_description("正在写入中！")
                    else:
                        for t in tqdm(self.text):
                            writer.writerows(t)
                            self.debug(t)
                            self.record(t)
                            tqdm(self.text).set_description("正在写入中！")
                    print("写入完成！")

        elif isinstance(self.text, dict) or isinstance(self.text[0], dict):
            if mode == "r":
                with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode) as f:
                    reader = csv.reader(f)
                    for row in reader:
                        print(row)
                        self.record(row)
            else:
                with open(f"{self.path}/{self.filename}", mode=mode, encoding=encode, newline="") as f:
                    Dictwriter = csv.DictWriter(f, headers)
                    Dictwriter.writeheader()
                    if  isinstance(self.text[0], dict):
                        if is_single:
                            for d in tqdm(self.text):
                                Dictwriter.writerow(d)
                                self.debug(d)
                                self.record(d)
                                tqdm(self.text).set_description("正在写入中！")
                        else:
                            for d in tqdm(self.text):
                                Dictwriter.writerows(d)
                                self.debug(d)
                                self.record(d)
                                tqdm(self.text).set_description("正在写入中！")
                        print("写入完成！")
                    else:
                        if is_single:
                            Dictwriter.writerow(self.text)
                            print("写入完成！")
                        else:
                            for row in tqdm(self.text):
                                Dictwriter.writerows(row)
                                self.record(row)
                                self.debug(row)
                                tqdm(self.text).set_description("正在写入中！")
                            print("写入完成！")

        else:
            print("TypeErorr: text must be list or dict !")


    """
    Save to your Excel !
    This function have a necessary arg to sheet_name .
    of couser, you also give some choice args, for example, mode to read and write, 
    excel that row, column and value,
    you even can change is_single that deal most datas !
    """
    def ente_excel(func) -> all:
        def wrapper(self, *args, **kwargs):
            func(self, *args, **kwargs)
        return wrapper

    @ente_excel
    def to_excel(
        self, sheet_name: str, mode="w", row=None, column=None, value=None, own=False, is_single=False
    ) -> None:
        wb = openpyxl.Workbook()
        sh = wb.create_sheet(sheet_name)
        if mode == "w":
            if not isinstance(self.text[0], Iterable):
                if own:
                    row, column, value = self.text
                    self.debug(row+"--"+column+"--"+value)
                    self.record(row+"--"+column+"--"+value)
                    sh.cell(row=row, column=column, value=value)
                    wb.save( f"{self.path}/{self.filename}" )
                    wb.close()
                    print("保存成功！")
                    return
                else:
                    tqdm_text = tqdm(self.text)
                    for t in tqdm_text:
                        row, column, value = t
                        self.debug(row+"--"+column+"--"+value)
                        self.record(row+"--"+column+"--"+value)
                        sh.cell(row=row, column=column, value=value)
                        tqdm_text.set_description("正在保存中！")
                    wb.save( f"{self.path}/{self.filename}" )
                    wb.close()
                    print("保存成功！")
                    return

        wb = openpyxl.load_workbook(f"{self.path}/{self.filename}")
        sh = wb[sheet_name]
        #按行读取数据转化为列表
        rows_data = list(sh.rows)
        self.record(rows_data)
        self.debug(rows_data)
        # 获取表单的表头信息
        titles = []
        for title in rows_data[0]:
            titles.append(title.value)
            self.warn("title.value："+title.value)
            self.record("title.value："+title.value)
        cases = []
        for case in rows_data[1:]:
            data = []
            for cell in case: 
                data.append(cell.value)
                if isinstance(cell.value,str):
                    data.append(cell.value)
                    self.warn("cell.value："+cell.value)
                    self.record("cell.value："+cell.value)
                else:
                    data.append(cell.value)
                    self.warn("cell.value："+cell.value)
                    self.record("cell.value："+cell.value)
                case_data = dict(  list (zip(titles,data) )  )
                cases.append(case_data)
                self.warn("case_data："+case_data)
                self.record("case_data："+case_data)
        return cases


    def to_db(self, db_type: str, login_data: dict, sql=None) -> all:
        # mysql
        if db_type == "mysql": 
            db = pymysql.connect(
                host=login_data["host"], 
                port=login_data["port"],
                user=login_data["user"],
                password=login_data["password"],
                database=login_data["database"],
                charset=login_data["charset"]
            )
            cursor = db.cursor()
            if sql:
                SqlSCInsert = sql
                try:
                    cursor.execute(SqlSCInsert)
                    # 提交到数据库执行
                    db.commit()
                    print("完成！")
                except Exception as e:
                    raise e
                # 如果发生错误则回滚
                    db.rollback()
                cursor.close()
                db.close()
            else:
                return db

        # mongodb
        elif db_type == "mongodb":
            try:
                client = pymongo.MongoClient(login_data["host"],  login_data["port"])
                mydb = login_data["database"]
                if login_data["password"]:
                    db = client.admin    # 连接系统默认数据库admin
                    db.authenticate("用户名", "密码")
                else:
                    db = client.mydb
                    myset = db.login_data['set']
                    print("\033[1;32m连接成功！\033[0m")
                    result = myset.sql
                    return result
            except Exception as e:
                print("\033[1;31m连接失败！\033[0m")
                print(f"错误：{e}")
            
        elif db_type == "redis":
            if not login_data:
                try:
                    conn =redis.Redis(
                        host=login_data["host"],
                        port=login_data["port"],
                        password=login_data["password"]
                    )
                    print("\033[1;32m连接成功！\033[0m")
                except Exception as e:
                    print("\033[1;31m连接失败！\033[0m")
                    print(f"错误：{e}")
            else:
                try:
                    conn =redis.Redis(
                        host=login_data["host"],
                        port=login_data["port"],
                        db=login_data.get("database", 0)
                    )
                    print("\033[1;32m连接成功！\033[0m")
                except Exception as e:
                    print("\033[1;31m连接失败！\033[0m")
                    print(f"错误：{e}")
            return conn




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

    
setuptools.setup(
    name="zjwbox",
    version = "0.3.2",
    author = "zjw",
    author_email="2415528031@qq.com",
    description="a box for package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://zjw.show",
    packages=setuptools.find_packages(),
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        "wheel", 
        "wget==3.2",
        "lxml==4.6.1", 
        "eventlet==0.29.1",
        "dataclasses",
        "pyppeteer",
        "selenium",
        "nest_asyncio==1.4.1",
        "parsel", 
        "requests", 
        "faker", 
        "openpyxl", 
        "pymongo", 
        "pymysql",
        "redis",
        "jsonpath",
        "pandas",
        "numpy",
        "aiohttp",
        "psutil",
        "fake_useragent",
        "pyquery",
        "DecryptLogin"
    ]

)



import pandas as pd
import numpy as np


class KnnMiss(object):
    def __init__(self, datas):
        self.datas = datas
        
    def get_miss(self):
        result = []
        miss_val = 0
        for data in self.datas:
            miss_data = self.datas[data].isnull()
            for i in miss_data:
                if i:
                    miss_val += 1
            print(f"{data}缺失{miss_val}个值！",end="")
            if miss_val:
                result.append(data)
            miss_val = 0
        if result is []:
            print("无缺失值！")

        return result

    def knn(self, miss_val):
        na_index=self.datas[self.datas[miss_val].isnull()].index.values
        normal_index = self.datas[~self.datas[miss_val].isnull()].index.values
        train_x = self.datas.iloc[normal_index]
        train_y = self.datas.iloc[normal_index,10]
        test_x = self.datas.iloc[na_index]
        test_y = self.datas.iloc[na_index,10]
        list(enumerate(train_x.iloc[:,2:3].values))
        distances = []
        for item in test_x.iloc[:,2:3].values: 
            dist = {}
            for index, item1 in enumerate(train_x.iloc[:,2:3].values):  
                distance = np.sqrt(np.sum(np.square(item - item1)))  
                dist[index] = distance 
            distances.append(dist)

        sorted_list = [sorted(item.items(), key = lambda x: x[1])[0:15] for item in distances]
        index_list =  [list(zip(*item_list))[0] for item_list in sorted_list]

        predict_var= {}
        for item in zip(na_index, index_list):
            predict_var[item[0]] = train_y.iloc[list(item[1])].mode()[0]

        self.datas[miss_val].fillna(predict_var,inplace=True)
    

    def deal_knn(self, miss_vals = None):
        miss_vals = self.get_miss()
        for miss_val in miss_vals:
            self.knn(miss_val)
        print(self.datas[1:11])    
        
        


# This is a example !
# if __name__ == "__main__":

#     datas = pd.read_csv("filepath")
#     knn = KnnMiss(datas)
#     knn.deal_knn()



# coding:utf-8
import requests
from hashlib import md5


class Chaojiying_Client(object):

    def __init__(self, username, password, soft_id):
        self.username = username
        password = password.encode('utf8')

        self.password = md5(password).hexdigest()
        self.soft_id = soft_id
        self.base_params = {
            'user': self.username,
            'pass2': self.password,
            'softid': self.soft_id,
        }
        self.headers = {
            'Connection': 'Keep-Alive',
            'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)',
        }


    def PostPic(self, im, codetype):
        """
        im: 图片字节
        codetype: 题目类型 参考 http://www.chaojiying.com/price.html
        """
        params = {
            'codetype': codetype,
        }
        params.update(self.base_params)
        files = {'userfile': ('ccc.jpg', im)}
        r = requests.post('http://upload.chaojiying.net/Upload/Processing.php', data=params, files=files,
                          headers=self.headers)
        return r.json()


    def ReportError(self, im_id):
        """
        im_id:报错题目的图片ID
        """
        params = {
            'id': im_id,
        }
        params.update(self.base_params)
        r = requests.post('http://upload.chaojiying.net/Upload/ReportError.php', data=params, headers=self.headers)
        return r.json()


def cjy_proof(image_path: str, image_type:str, login_args: dict):
    user, password, proof_type = login_args["user"], login_args["password"], login_args["proof_type"]
    chaojiying = Chaojiying_Client(user, password, proof_type)  # 用户中心>>软件ID 生成一个替换 96001
    im = open(image_path, 'rb').read()  # 本地图片文件路径 来替换 a.jpg 有时WIN系统须要//
    all_str = chaojiying.PostPic(im, image_type)  # 1902 验证码类型  官方网站>>价格体系 3.4+版 print 后要加()
    pic_str = all_str['pic_str']
    result = pic_str.replace('|', ',')
    return result



# if __name__ == "__main__":
#     login_args = {
#         "user": "",
#         "password": "",
#         "proof_type": ''
#     }
#     a=cjy_proof(image_path=r'G:\proof_imag\02.jpg.jpg',image_type=9004, login_args=login_args)
#     print(a)





import re


def CheckUrl(url: str) -> all:
    register_domain = [
    "lagou",
    "douban",
    "taobao",
    "zhihu",
    "jingdong",
    "baidupan",
    "QQZone",
    "ifeng",
    "eSurfing",
    "zgconline",
    "github",
    "zt12306",
    "music163",
    "sohu",
    "bilibili",
    "weibo",
    "youdao",
    "renren"
    ]
    pattern = r"[http, https]://\w+.(.*?).com"
    result = re.findall(string=url, pattern=pattern)
    if result[0] in register_domain:
        return True
    else:
        return False




import requests, re, wget, zipfile, os
from pprint import pprint


def update_driver(os_type: str="win32", file_path: str=None, zip_path: str=None, del_zip: str=False) -> None:
    url = "http://chromedriver.storage.googleapis.com/?delimiter=/&prefix="
    session = requests.Session()
    resp = session.get(url)
    text = resp.text

    pattern = r"<Prefix>(\d.*?)/"
    versions = re.findall(pattern=pattern, string=text)
    pprint(versions)
    print(f"一共找到{len(versions)}个版本，最新版本{versions[-1]}")
    while True:
        change_version = input("请选择要更新的版本：")
        if change_version in versions:
            print(f"已选择：{change_version} !")
            break
        else:
            print(f"无该版本：{change_version}，请重新输入！")

    if not zip_path:
        zip_path = "chromedriver.zip"
    wget.download(f"http://chromedriver.storage.googleapis.com/{change_version}/chromedriver_{os_type}.zip", out=zip_path)
    print(f"zip文件保存成功！路径：{os.getcwd()}{os.path.sep}{zip_path}")
    zfile = zipfile.ZipFile(zip_path, "r")
    if file_path:
        zfile.extractall(path=file_path)
    else:
        zfile.extractall()
    zfile.close()
    if del_zip:
        os.remove(os.getcwd() + f"{os.path.sep}{zip_path}")
        print(f"zip文件已删除！")
    print(f"exe文件保存成功！路径：{file_path}" + f"{os.path.sep}" + "chromedriver.exe")




__name__ = ["update_driver"]

if __name__ == "__main__":
    update_driver(del_zip=True, file_path="D:\\test")